#include <STC8G.H>
#include "define.h"
#include <intrins.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#define		RXD		P30
#define		TXD		P31
#define		SCL		P32
#define		SDA		P33
#define		FOSC		11059200UL
#define		BAUD		115200UL
#define		BRT			(0x10000-FOSC/BAUD/4)
#define		T_Buffer_Len	64	//Uart1ͻ泤
#define		R_Buffer_Len	64	//Uart1ջ泤
#define		AS5600_Write	0x6C
#define		AS5600_Read		0x6D

unsigned char		RP;							//Uart1ָ
unsigned char		TP;							//Uart1ָ
unsigned char		Uart_Send_Lenth;			//Uart1ͳ
unsigned char xdata	R_Buffer[R_Buffer_Len];		//Uart1ջ
unsigned char xdata	T_Buffer[T_Buffer_Len];		//Uart1ͻ
bit I2C_Busy;

/*----------------------------ʱ10us@STC-Y6@11.0592MHz----------------------------*/
void Delay_10us(void)
{
	unsigned char i;
	i=35;
	while(--i);
}

/*----------------------------ʱx10us----------------------------*/
void Delay_x10us(unsigned char x)
{
	while(x--)
		Delay_10us();
}

/*----------------------------ʱ10ms@STC-Y6@11.0592MHz----------------------------*/
void Delay_10ms(void)
{
	unsigned char i,j;
	_nop_();
	_nop_();
	i=144;
	j=157;
	do
	{
		while(--j);
	}while(--i);
}

/*----------------------------ʱx10ms----------------------------*/
void Delay_x10ms(unsigned char x)
{
	while(x--)
		Delay_10ms();
}

void UART_Send(unsigned int x)
{
	TP=0;
	Uart_Send_Lenth=x;
	TI=1;
}

void Uart_Printf(unsigned char *v,...)
{
	va_list ap;
	va_start(ap,v);
	while(Uart_Send_Lenth);
	UART_Send(vsprintf(T_Buffer,v,ap));
	va_end(ap);
}

void I2C_Start(void)
{
	I2C_Busy=1;
	I2CMSCR=0x81;
	while(I2C_Busy);
}

void I2C_SendData(unsigned char dat)
{
	I2CTXD=dat;
	I2C_Busy=1;
	I2CMSCR=0x82;
	while(I2C_Busy);
}

void I2C_RecvACK(void)
{
	I2C_Busy=1;
	I2CMSCR=0x83;
	while(I2C_Busy);
}

unsigned char I2C_RecvData(void)
{
	I2C_Busy=1;
	I2CMSCR=0x84;
	while(I2C_Busy);
	return I2CRXD;
}

void I2C_SendACK(void)
{
	I2CMSST=0x00;
	I2C_Busy=1;
	I2CMSCR=0x85;
	while(I2C_Busy);
}

void I2C_SendNAK(void)
{
	I2CMSST=0x01;
	I2C_Busy=1;
	I2CMSCR=0x85;
	while(I2C_Busy);
}

void I2C_Stop(void)
{
	I2C_Busy=1;
	I2CMSCR=0x86;
	while(I2C_Busy);
}

void AS5600_Write_One_Byte(unsigned char addr,unsigned char dat)
{
	I2C_Start();
	I2C_SendData(AS5600_Write);
	I2C_RecvACK();
	I2C_SendData(addr);
	I2C_RecvACK();
	I2C_SendData(dat);
	I2C_RecvACK();
	I2C_Stop();
}

void AS5600_Write_Two_Byte(unsigned char addr,unsigned int dat)
{
	I2C_Start();
	I2C_SendData(AS5600_Write);
	I2C_RecvACK();
	I2C_SendData(addr);
	I2C_RecvACK();
	I2C_SendData((unsigned char)(dat>>8));
	I2C_RecvACK();
	I2C_SendData((unsigned char)(dat&0x00FF));
	I2C_RecvACK();
	I2C_Stop();
}

unsigned char AS5600_Read_One_Byte(unsigned char addr)
{
	unsigned char dat;
	I2C_Start();
	I2C_SendData(AS5600_Write);
	I2C_RecvACK();
	I2C_SendData(addr);
	I2C_RecvACK();
	I2C_Start();
	I2C_SendData(AS5600_Read);
	I2C_RecvACK();
	dat=I2C_RecvData();
	I2C_SendNAK();
	I2C_Stop();
	return dat;
}

unsigned int AS5600_Read_Two_Byte(unsigned char addr)
{
	unsigned char msb,lsb;
	unsigned int dat;
	I2C_Start();
	I2C_SendData(AS5600_Write);
	I2C_RecvACK();
	I2C_SendData(addr);
	I2C_RecvACK();
	I2C_Start();
	I2C_SendData(AS5600_Read);
	I2C_RecvACK();
	msb=I2C_RecvData();
	I2C_SendACK();
	lsb=I2C_RecvData();
	I2C_SendNAK();
	I2C_Stop();
	dat=(unsigned int)msb<<8;
	dat|=(unsigned int)lsb;
	return dat;
}

void AS5600_Init(void)
{
	AS5600_Write_Two_Byte(0x01,0x00);
	Delay_x10us(100);
	AS5600_Write_Two_Byte(0x03,0x00);
	Delay_x10us(100);
	AS5600_Write_One_Byte(0x07,0x00);
	Delay_x10us(100);
	AS5600_Write_One_Byte(0x08,0x00);
	Delay_x10us(100);
//	AS5600_Write_One_Byte(0xFF,0x80);	//棺˲棡
//	Delay_x10us(100);
	Uart_Printf("ZMCO=%BU\r\n",AS5600_Read_One_Byte(0x00));
	Uart_Printf("ZPOS=%U\r\n",AS5600_Read_Two_Byte(0x01));
	Uart_Printf("MPOS=%U\r\n",AS5600_Read_Two_Byte(0x03));
	Uart_Printf("MANG=%U\r\n",AS5600_Read_Two_Byte(0x05));
	Uart_Printf("CONF=0x%02BX%02BX\r\n",AS5600_Read_One_Byte(0x07),AS5600_Read_One_Byte(0x08));
}

void Init(void)
{
	P_SW2|=EAXFR;
	
	P3M0=0x00;
	P3M1=0x00;
	P5M0=0x00;
	P5M1=0x00;
	P3PU=0x0c;
	
	AUXR=0x40;		//öʱ0ʱΪ12Tģʽöʱ1Ϊ1Tģʽöʱ1Ϊʷ
	TMOD=0x01;		//öʱ0Ϊ16λԶװģʽöʱ1Ϊ16λԶװģʽ
	TL0=0x00;		//öʱ0ʼֵ5ms
	TH0=0xEE;		//öʱ0ʼֵ5ms
	TF0=0;			//TF0жϱ־λ
	ET0=1;			//öʱ0ж
	
	SCON=0x50;		//UART1ģʽΪ8λݿɱ䲨
	TL1=BRT;		//UART1
    TH1=BRT>>8;		//UART1
	TR1=1;			//򿪶ʱ1
	ES=1;			//UART1ж
	
	I2CCFG=0xC6;	//345.6K@11.0592M
	I2CMSCR=EMSI;
	I2CMSST=0x00;
	
	EA=1;			//ж
	
	AS5600_Init();
}

void main(void)
{
	Init();
	while(1)
	{
		Uart_Printf("Ƕ=%04U =%04U ⵽=%BU ų̫=%BU ų̫ǿ=%BU =%03BU =%04U\r\n",AS5600_Read_Two_Byte(0x0C),AS5600_Read_Two_Byte(0x0E),(AS5600_Read_One_Byte(0x0B)&0x20)>>5,(AS5600_Read_One_Byte(0x0B)&0x10)>>4,(AS5600_Read_One_Byte(0x0B)&0x08)>>3,AS5600_Read_One_Byte(0x1A),AS5600_Read_Two_Byte(0x1B));
		Delay_x10ms(2);
	}
}

void Uart_Start(void)
{
	TL0=0x00;
	TH0=0xEE;
	TR0=1;
}

void Uart_Stop(void)
{
	TR0=0;
	TL0=0x00;
	TH0=0xEE;
	RP=0;
	memset(R_Buffer,0x00,sizeof R_Buffer);
}

void Timer0_Isr(void) interrupt 1 
{
	Uart_Stop();
}

void Uart_Isr(void) interrupt 4
{
	if(RI)
	{
		RI=0;
		Uart_Start();
		R_Buffer[RP]=SBUF;
		if(RP==0)
		{
			IAP_CONTR=0x60;
		}
		if(RP==R_Buffer_Len-1)
		{
			Uart_Stop();
		}
		else if(TR0)
		{
			RP++;
		}
	}
	if(TI)
	{
		TI=0;
		if(Uart_Send_Lenth!=0)
		{
			SBUF=(T_Buffer[TP]);
			TP++;
		}
		if(TP==Uart_Send_Lenth)
		{
			TP=0;
			Uart_Send_Lenth=0;
		}
	}
}

void I2C_Isr(void) interrupt 24
{
	_push_(P_SW2);
	P_SW2|=EAXFR;
	if(I2CMSST&MSIF)
	{
		I2CMSST&=~MSIF;
		I2C_Busy=0;
	}
	_pop_(P_SW2);
}